/************************************************************

XTEA-1 by Tom St Denis implemented by Alexander Myasnikov

Ultrafast secured edition of TEA

Web: www.darksoftware.narod.ru

************************************************************/


#include <stdlib.h>

typedef unsigned long u32;
typedef long int s32;

u32 key[4];


typedef struct
{

  u32 sum1;
  u32 key1;
  u32 sum2;
  u32 key2;

} pref_rec;

static pref_rec prefetch[32];




void __stdcall __export
setup (u32 * data)
{
  u32 r, sum = 0;

  key[0] = data[0];
  key[1] = data[1];
  key[2] = data[2];
  key[3] = data[3];


  for (r = 0; r < 32; r++)

    {
      prefetch[r].sum1 = sum;
      prefetch[r].key1 = key[sum & 3];
      sum += 0x9E3779B9;
      prefetch[r].sum2 = sum;
      prefetch[r].key2 = key[(sum >> 11) & 3];

    }

}

void __stdcall __export
crypt (u32 * plain)
{
  register u32 y, z;

  y = plain[0] + key[0];
  z = plain[1] + key[1];

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[0].sum1) + _lrotl (prefetch[0].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[0].sum2) + _lrotl (prefetch[0].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[1].sum1) + _lrotl (prefetch[1].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[1].sum2) + _lrotl (prefetch[1].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[2].sum1) + _lrotl (prefetch[2].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[2].sum2) + _lrotl (prefetch[2].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[3].sum1) + _lrotl (prefetch[3].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[3].sum2) + _lrotl (prefetch[3].key2,
							     y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[4].sum1) + _lrotl (prefetch[4].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[4].sum2) + _lrotl (prefetch[4].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[5].sum1) + _lrotl (prefetch[5].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[5].sum2) + _lrotl (prefetch[5].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[6].sum1) + _lrotl (prefetch[6].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[6].sum2) + _lrotl (prefetch[6].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[7].sum1) + _lrotl (prefetch[7].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[7].sum2) + _lrotl (prefetch[7].key2,
							     y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[8].sum1) + _lrotl (prefetch[8].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[8].sum2) + _lrotl (prefetch[8].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[9].sum1) + _lrotl (prefetch[9].key1,
							     z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[9].sum2) + _lrotl (prefetch[9].key2,
							     y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[10].sum1) +
    _lrotl (prefetch[10].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[10].sum2) +
    _lrotl (prefetch[10].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[11].sum1) +
    _lrotl (prefetch[11].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[11].sum2) +
    _lrotl (prefetch[11].key2, y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[12].sum1) +
    _lrotl (prefetch[12].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[12].sum2) +
    _lrotl (prefetch[12].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[13].sum1) +
    _lrotl (prefetch[13].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[13].sum2) +
    _lrotl (prefetch[13].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[14].sum1) +
    _lrotl (prefetch[14].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[14].sum2) +
    _lrotl (prefetch[14].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[15].sum1) +
    _lrotl (prefetch[15].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[15].sum2) +
    _lrotl (prefetch[15].key2, y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[16].sum1) +
    _lrotl (prefetch[16].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[16].sum2) +
    _lrotl (prefetch[16].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[17].sum1) +
    _lrotl (prefetch[17].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[17].sum2) +
    _lrotl (prefetch[17].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[18].sum1) +
    _lrotl (prefetch[18].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[18].sum2) +
    _lrotl (prefetch[18].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[19].sum1) +
    _lrotl (prefetch[19].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[19].sum2) +
    _lrotl (prefetch[19].key2, y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[20].sum1) +
    _lrotl (prefetch[20].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[20].sum2) +
    _lrotl (prefetch[20].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[21].sum1) +
    _lrotl (prefetch[21].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[21].sum2) +
    _lrotl (prefetch[21].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[22].sum1) +
    _lrotl (prefetch[22].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[22].sum2) +
    _lrotl (prefetch[22].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[23].sum1) +
    _lrotl (prefetch[23].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[23].sum2) +
    _lrotl (prefetch[23].key2, y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[24].sum1) +
    _lrotl (prefetch[24].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[24].sum2) +
    _lrotl (prefetch[24].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[25].sum1) +
    _lrotl (prefetch[25].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[25].sum2) +
    _lrotl (prefetch[25].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[26].sum1) +
    _lrotl (prefetch[26].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[26].sum2) +
    _lrotl (prefetch[26].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[27].sum1) +
    _lrotl (prefetch[27].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[27].sum2) +
    _lrotl (prefetch[27].key2, y);


  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[28].sum1) +
    _lrotl (prefetch[28].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[28].sum2) +
    _lrotl (prefetch[28].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[29].sum1) +
    _lrotl (prefetch[29].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[29].sum2) +
    _lrotl (prefetch[29].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[30].sum1) +
    _lrotl (prefetch[30].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[30].sum2) +
    _lrotl (prefetch[30].key2, y);

  y +=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[31].sum1) +
    _lrotl (prefetch[31].key1, z);
  z +=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[31].sum2) +
    _lrotl (prefetch[31].key2, y);

  plain[0] = y ^ key[2];
  plain[1] = z ^ key[3];
}



void __stdcall __export
decrypt (u32 * plain)
{
  register u32 y, z;

  y = plain[0] ^ key[2];
  z = plain[1] ^ key[3];

  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[31].sum2) +
    _lrotl (prefetch[31].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[31].sum1) +
    _lrotl (prefetch[31].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[30].sum2) +
    _lrotl (prefetch[30].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[30].sum1) +
    _lrotl (prefetch[30].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[29].sum2) +
    _lrotl (prefetch[29].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[29].sum1) +
    _lrotl (prefetch[29].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[28].sum2) +
    _lrotl (prefetch[28].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[28].sum1) +
    _lrotl (prefetch[28].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[27].sum2) +
    _lrotl (prefetch[27].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[27].sum1) +
    _lrotl (prefetch[27].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[26].sum2) +
    _lrotl (prefetch[26].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[26].sum1) +
    _lrotl (prefetch[26].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[25].sum2) +
    _lrotl (prefetch[25].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[25].sum1) +
    _lrotl (prefetch[25].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[24].sum2) +
    _lrotl (prefetch[24].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[24].sum1) +
    _lrotl (prefetch[24].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[23].sum2) +
    _lrotl (prefetch[23].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[23].sum1) +
    _lrotl (prefetch[23].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[22].sum2) +
    _lrotl (prefetch[22].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[22].sum1) +
    _lrotl (prefetch[22].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[21].sum2) +
    _lrotl (prefetch[21].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[21].sum1) +
    _lrotl (prefetch[21].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[20].sum2) +
    _lrotl (prefetch[20].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[20].sum1) +
    _lrotl (prefetch[20].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[19].sum2) +
    _lrotl (prefetch[19].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[19].sum1) +
    _lrotl (prefetch[19].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[18].sum2) +
    _lrotl (prefetch[18].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[18].sum1) +
    _lrotl (prefetch[18].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[17].sum2) +
    _lrotl (prefetch[17].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[17].sum1) +
    _lrotl (prefetch[17].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[16].sum2) +
    _lrotl (prefetch[16].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[16].sum1) +
    _lrotl (prefetch[16].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[15].sum2) +
    _lrotl (prefetch[15].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[15].sum1) +
    _lrotl (prefetch[15].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[14].sum2) +
    _lrotl (prefetch[14].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[14].sum1) +
    _lrotl (prefetch[14].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[13].sum2) +
    _lrotl (prefetch[13].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[13].sum1) +
    _lrotl (prefetch[13].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[12].sum2) +
    _lrotl (prefetch[12].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[12].sum1) +
    _lrotl (prefetch[12].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[11].sum2) +
    _lrotl (prefetch[11].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[11].sum1) +
    _lrotl (prefetch[11].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[10].sum2) +
    _lrotl (prefetch[10].key2, y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[10].sum1) +
    _lrotl (prefetch[10].key1, z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[9].sum2) + _lrotl (prefetch[9].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[9].sum1) + _lrotl (prefetch[9].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[8].sum2) + _lrotl (prefetch[8].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[8].sum1) + _lrotl (prefetch[8].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[7].sum2) + _lrotl (prefetch[7].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[7].sum1) + _lrotl (prefetch[7].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[6].sum2) + _lrotl (prefetch[6].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[6].sum1) + _lrotl (prefetch[6].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[5].sum2) + _lrotl (prefetch[5].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[5].sum1) + _lrotl (prefetch[5].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[4].sum2) + _lrotl (prefetch[4].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[4].sum1) + _lrotl (prefetch[4].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[3].sum2) + _lrotl (prefetch[3].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[3].sum1) + _lrotl (prefetch[3].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[2].sum2) + _lrotl (prefetch[2].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[2].sum1) + _lrotl (prefetch[2].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[1].sum2) + _lrotl (prefetch[1].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[1].sum1) + _lrotl (prefetch[1].key1,
							     z);
  z -=
    ((y << 4) ^ (y >> 5)) + (y ^ prefetch[0].sum2) + _lrotl (prefetch[0].key2,
							     y);
  y -=
    ((z << 4) ^ (z >> 5)) + (z ^ prefetch[0].sum1) + _lrotl (prefetch[0].key1,
							     z);

  plain[0] = y - key[0];
  plain[1] = z - key[1];
}
